@@ -586,10 +586,16 @@ def wrap_object(module, name, factory, args=(), kwargs={}):
 # creating decorators see the decorator decorator instead.
 
 def function_wrapper(wrapper):
-    @functools.wraps(wrapper)
-    def _wrapper(wrapped):
-        return FunctionWrapper(wrapped, wrapper)
-    return _wrapper
+    def _wrapper(wrapped, instance, args, kwargs):
+        target_wrapped = args[0]
+        if instance is None:
+            target_wrapper = wrapper
+        elif inspect.isclass(instance):
+            target_wrapper = wrapper.__get__(None, instance)
+        else:
+            target_wrapper = wrapper.__get__(instance, type(instance))
+        return FunctionWrapper(target_wrapped, target_wrapper)
+    return FunctionWrapper(wrapper, _wrapper)
 
 def wrap_function_wrapper(module, name, wrapper):
     return wrap_object(module, name, FunctionWrapper, (wrapper,))
@@ -601,16 +607,24 @@ def patch_function_wrapper(module, name):
 
 def transient_function_wrapper(module, name):
     def _decorator(wrapper):
-        @function_wrapper
         def _wrapper(wrapped, instance, args, kwargs):
-            try:
+            target_wrapped = args[0]
+            if instance is None:
+                target_wrapper = wrapper
+            elif inspect.isclass(instance):
+                target_wrapper = wrapper.__get__(None, instance)
+            else:
+                target_wrapper = wrapper.__get__(instance, type(instance))
+            def _execute(wrapped, instance, args, kwargs):
                 (parent, attribute, original) = resolve_path(module, name)
-                replacement = FunctionWrapper(original, wrapper)
+                replacement = FunctionWrapper(original, target_wrapper)
                 setattr(parent, attribute, replacement)
-                return wrapped(*args, **kwargs)
-            finally:
-                setattr(parent, attribute, original)
-        return _wrapper
+                try:
+                    return wrapped(*args, **kwargs)
+                finally:
+                    setattr(parent, attribute, original)
+            return FunctionWrapper(target_wrapped, _execute)
+        return FunctionWrapper(wrapper, _wrapper)
     return _decorator
 
 # A weak function proxy. This will work on instance methods, class
